#!/usr/bin/perl
#Yongsheng Bai 6-14-2010 (python scripts are taken from ERANGE (Mortazavi et al., 2008)) 
#perl ERANGE_pipeline_RCMI2010.pl input_sequence_path input_sequence_file output_files_path 

#i.e. perl ERANGE_pipeline_RCMI2010.pl /ccmb/CoreBA/Projects/RCMI2010/private s_6_24A-WT_sequence.txt /ccmb/CoreBA/Projects/RCMI2010/private
#i.e. perl ERANGE_pipeline_RCMI2010.pl /ccmb/CoreBA/Projects/RCMI2010/private s_7_24A-KO_sequence.txt /ccmb/CoreBA/Projects/RCMI2010/private

use strict;

my ($path,$seq_file,$bowtie_path)= @ARGV;

my $temp_file = system("find $path -name $seq_file > file_location");
if (-z("file_location"))#check_file exists in the directory 
{
  print "Original $seq_file is not available under $path, please check your path!\n";
}
else 
{
  print "---------------- 1 --------------\n";
  print "Start bowtie alignment step......\n";
  system("bowtie /ccmb/CoreBA/Projects/Data/mouse/mm9sp37_indexes -v 2 -k 11 -m 10 --best -p 5 -q $path/$seq_file --un $bowtie_path/$seq_file.unmapped.fa --max $bowtie_path/$seq_file.repeat.fa $bowtie_path/$seq_file.bowtie.txt");
  print "Bowtie alignment step is done!\n\n";

  print "---------------- 2 --------------\n";
  print "Start making rds file fom bowtie step......\n";
  my $label = substr($seq_file,0,-4);
  system("python2.6 /usr/local/bioinf/ERANGE3.1/commoncode/makerdsfrombowtie.py $label $bowtie_path/$seq_file.bowtie.txt /data/ybai/tmp/$seq_file.rds -RNA /ccmb/CoreBA/Projects/Data/mouse/knownGene.txt -index");
  print "Making rds file step is done!\n\n";

  print "---------------- 3 --------------\n";
  print "Start changing database cache size step......\n";
  system("python2.6 /usr/local/bioinf/ERANGE3.1/commoncode/rdsmetadata.py /data/ybai/tmp/$seq_file.rds -defaultcache 6000000");
  print "Changing database cache size step is done!\n\n";

  print "---------------- copying rds file to the memory --------------\n\n";
  system("cp /data/ybai/tmp/$seq_file.rds /dev/shm");

  print "---------------- 4 --------------\n";
  print "Start counting unique reads falling on gene model step......\n";
  system("python2.6 /usr/local/bioinf/ERANGE3.1/commoncode/geneMrnaCounts.py mouse /dev/shm/$seq_file.rds /data/ybai/tmp/$seq_file.uniqs.count -markNM");
  print "Counting unique reads falling on gene model step is done!\n\n";
  
  print "---------------- 5 --------------\n";
  print "Start counting splice reads step......\n";
  system("python2.6 /usr/local/bioinf/ERANGE3.1/commoncode/geneMrnaCounts.py mouse /dev/shm/$seq_file.rds /data/ybai/tmp/$seq_file.splices.count -splices -noUniqs");
  print "Counting splice reads step is done!\n\n";

  print "---------------- 6 --------------\n";
  print "Start calculating a first-pass RPKM step......\n";
  system("python2.6 /usr/local/bioinf/ERANGE3.1/commoncode/normalizeExpandedExonic.py mouse /dev/shm/$seq_file.rds /data/ybai/tmp/$seq_file.uniqs.count none /data/ybai/tmp/$seq_file.firstpass.rpkm");
  print "Calculating a first-pass RPKM step is done!\n\n";

  print "---------------- 7 --------------\n";
  print "Start recounting the unique reads step......\n";
  system("python2.6 /usr/local/bioinf/ERANGE3.1/commoncode/geneMrnaCountsWeighted.py mouse /dev/shm/$seq_file.rds /data/ybai/tmp/$seq_file.firstpass.rpkm /data/ybai/tmp/$seq_file.uniqs.recount -uniq");
  print "Recounting the unique reads step is done!\n\n";

  print "---------------- 8 --------------\n";
  print "Start finding new regions outside of gene models step......\n";
  system("python2.6 /usr/local/bioinf/ERANGE3.1/commoncode/findall.py $label /dev/shm/$seq_file.rds /data/ybai/tmp/$seq_file.newregions.txt -raw -minimum 25 -spacing 40 -nodirectionality -notrim -noshift -flag NM");
  print "Finding new regions outside of gene models step is done!\n\n";
  
  print "---------------- 9 --------------\n";
  print "Start filtering out new regions that overlap repeats more than a certain fraction step......\n";
  system("python2.6 /usr/local/bioinf/ERANGE3.1/commoncode/checkrmask.py /ccmb/CoreBA/Projects/Data/mouse/mm9repeats/rmask.db /data/ybai/tmp/$seq_file.newregions.txt /data/ybai/tmp/$seq_file.newregions.repstatus /data/ybai/tmp/$seq_file.newregions.good -startField 1");
  print "Filtering out new regions that overlap repeats more than a certain fraction step is done!\n\n";

  print "---------------- 10 --------------\n";
  print "Start mapping all candidate regions that are within a 20kb radius of a gene step......\n";
  system("python2.6 /usr/local/bioinf/ERANGE3.1/commoncode/getallgenes.py mouse /data/ybai/tmp/$seq_file.newregions.good /data/ybai/tmp/$seq_file.candidates.txt 20001 -trackfar -cache");
  print "Mapping all candidate regions that are within a 20kb radius of a gene step is done!\n\n";
  
  print "---------------- 11 --------------\n";
  print "Start calculating expanded exonic read density step......\n";
  system("python2.6 /usr/local/bioinf/ERANGE3.1/commoncode/normalizeExpandedExonic.py mouse /dev/shm/$seq_file.rds /data/ybai/tmp/$seq_file.uniqs.recount /data/ybai/tmp/$seq_file.splices.count /data/ybai/tmp/$seq_file.expanded.rpkm /data/ybai/tmp/$seq_file.candidates.txt /data/ybai/tmp/$seq_file.accepted.rpkm");
  print "Calculating expanded exonic read density step is done!\n\n";

  print "---------------- 12 --------------\n";
  print "Start creating bed file of accepted candidate regions step......\n";
  system("python2.6 /usr/local/bioinf/ERANGE3.1/commoncode/regiontobed.py $label /data/ybai/tmp/$seq_file.accepted.rpkm /data/ybai/tmp/$seq_file.bed -color 255,0,0");
  print "Creating bed file of accepted candidate regions step is done!\n\n";

  print "---------------- 13 --------------\n";
  print "Start weighing multi-reads step......\n";
  system("python2.6 /usr/local/bioinf/ERANGE3.1/commoncode/geneMrnaCountsWeighted.py mouse /dev/shm/$seq_file.rds /data/ybai/tmp/$seq_file.expanded.rpkm /data/ybai/tmp/$seq_file.multi.count -accept /data/ybai/tmp/$seq_file.accepted.rpkm -multi");
  print "Weighing multi-reads step is done!\n\n";

  print "---------------- 14 --------------\n";
  print "Start calculating final exonic read density step......\n";
  system("python2.6 /usr/local/bioinf/ERANGE3.1/commoncode/normalizeFinalExonic.py /dev/shm/$seq_file.rds /data/ybai/tmp/$seq_file.expanded.rpkm /data/ybai/tmp/$seq_file.multi.count /data/ybai/tmp/$seq_file.final.rpkm");
  print "Calculating final exonic read density step is done!\n\n";

  print "---------------- Happy results!!! --------------\n";
  print "Start moving all files to destination directory and clear up memory step......\n";
  system("mv /data/ybai/tmp/* $bowtie_path/");
  system("rm /dev/shm/$seq_file.rds");
  print "Moving all files to destination directory and clear up memory step is done!\n\n";
   
}

