####################################################################################
##  Authors: Maureen Sartor and Yongsheng Bai
##  Gene Level Analysis
##
####################################################################################
outpath <- Sys.getenv("pathToOutput")  #  Change to correct output path
path1<- Sys.getenv("pathToInput")   #  Change to correct input path

###  Read in data from lane 1
gene1<- read.delim(paste(path1,Sys.getenv("finalFile1"),sep=""),sep="\t",header=T)
###  Read in data from lane 2
gene2<- read.delim(paste(path1,Sys.getenv("finalFile2"),sep=""),sep="\t",header=T)

######  Read FAR info
far1 <- read.delim(paste(path1,Sys.getenv("acceptedFile1"),sep=""),sep="\t",header=F)
far2 <- read.delim(paste(path1,Sys.getenv("acceptedFile2"),sep=""),sep="\t",header=F)

far.togo<- rbind(cbind(lane="lane1",far1), cbind(lane="lane2",far2))
write.table(far.togo,file=paste(outpath,"ERANGE-FAR_Prediction_Results.txt",sep=""),sep="\t",col.names=TRUE,row.names=FALSE,quote=F)

############  Merge by identical gene symbols
symbols<-unique(c(as.character(gene1[,1]),as.character(gene2[,1]))) 
lane1<- gene1[match(symbols,gene1[,1]),]
lane2<- gene2[match(symbols,gene2[,1]),]

raw.togo<-cbind(lane1,lane2)
write.table(raw.togo,file=paste(outpath,"ERANGE-Raw-Genes.txt",sep=""),sep="\t",col.names=TRUE,row.names=FALSE,quote=F)

##############################################################
lrpkm1<-log2(lane1[,3]+0.0001)[!is.na(lane1[,3])&!is.na(lane2[,3])]
lrpkm2<-log2(lane2[,3]+0.0001)[!is.na(lane1[,3])&!is.na(lane2[,3])]
len<-rowMeans(cbind(lane1[,2],lane2[,2]),na.rm=T)
RPKM<-cbind(lane1[,3],lane2[,3])

norm1<- log2((lane1[,3]+0.0001)*len)[!is.na(lane1[,3])&!is.na(lane2[,3])]
norm2<- log2((lane2[,3]+0.0001)*len)[!is.na(lane1[,3])&!is.na(lane2[,3])]

l2a<- rowMeans(cbind(norm1,norm2))
l2r<- norm2 - norm1 
bmp(paste(outpath,"M-A plot.bmp",sep=""),width=800,height=800)
plot(l2a,norm2 - norm1,xlab="average log2(RPKM)",ylab="log-ratio",main="M-A plot")
dev.off()

##############################################################
lrpkm1[is.na(lrpkm1)]<-log2(0.0001)
lrpkm2[is.na(lrpkm2)]<-log2(0.0001)

lane1clean<-lane1[!is.na(lane1[,3])&!is.na(lane2[,3]),][!lrpkm1==lrpkm2,]
lane2clean<-lane2[!is.na(lane1[,3])&!is.na(lane2[,3]),][!lrpkm1==lrpkm2,]

norm1clean<-norm1[!norm1==norm2]
norm2clean<-norm2[!norm1==norm2]

symbol<-symbols[!is.na(lane1[,3])&!is.na(lane2[,3])][!norm1==norm2]
gene.len<-len[!is.na(lane1[,3])&!is.na(lane2[,3])][!norm1==norm2]
rpkm.togo<-RPKM[!is.na(lane1[,3])&!is.na(lane2[,3]),][!norm1==norm2,]

l2a<-rowMeans(cbind(norm1,norm2))
l2aclean<-rowMeans(cbind(norm1clean,norm2clean))

scatter.smooth(l2aclean,log(abs(norm2clean-norm1clean)),xlab="average log2(normalized reads)",
	ylab="Absolute log-Difference",main="Estimating variance as function of normalized reads",col="blue", span=0.4)
vfit <- loess(log(abs(norm2clean-norm1clean)) ~ l2aclean, span=0.4)  #2/3)
fitted<-vfit$fitted

s02<-exp(log(1/2)+2*fitted + digamma(400000/2) - digamma(1/2) - log(400000/1))  # Using 400000 to represent approaching infinity

bmp(paste(outpath,"AbsM-A plot.bmp",sep=""),width=800,height=800)
scatter.smooth(l2a[!lrpkm1==lrpkm2],log(abs(lrpkm2[!lrpkm1==lrpkm2]-lrpkm1[!lrpkm1==lrpkm2])),xlab="average log2(RPKM)",
	ylab="Absolute log-Difference",main="Estimating variance as function of RPKM",col="blue")
points(l2a[!lrpkm1==lrpkm2],log(sqrt(s02)),col="red")
dev.off()

zscore<- (norm2clean-norm1clean)/sqrt(s02)  
pval<- 2* pnorm(abs(zscore),lower.tail=F)
fdr<-p.adjust(pval,method="BH")

logfold<- norm2clean-norm1clean
fc<-ifelse(logfold>0,2^logfold,-1/(2^logfold))


########  Get Entrez Gene IDs
load("mouse_gene.info.RData")  
annot<-gene.info[match(symbol,gene.info[,2]),1:4]  ###  Match to symbol

to.go<- cbind(annot,symbol,gene.len,rpkm.togo,norm1clean,norm2clean,logfold,fc,zscore,pval,fdr)
colnames(to.go)<-c("EntrezID","symbol","cytoband","genename","X.gene","len_kb","RPKM1","RPKM2","norm_reads1","norm_reads2","logfold","fc",
	"zscore","pvalue","FDR")
write.table(to.go,file=paste(outpath,"ERANGE-Results-Genes.txt",sep=""),sep="\t",col.names=TRUE,row.names=FALSE,quote=F)

#bmp(paste(outpath,"Volcano_plot.bmp",sep=""),width=800,height=800)
#plot(logfold,-log10(pval),breaks=21,main="Volcano plot",ylab="-log10(p-value)",xlab="log2(fold change)")
#dev.off()

#select down-regulated genes only
to.go.clean<-to.go[!is.na(to.go[,1]),]
down.to.go.tmp<- to.go.clean[to.go.clean[,11]<=0,]
down.to.go<- down.to.go.tmp[!duplicated(down.to.go.tmp[,1]),]

#sort rows based on P value
sort.down.to.go<- down.to.go[order(down.to.go$pvalue),]

#take FDR<0.01 genes
fdr.sort.down.to.go<-sort.down.to.go[sort.down.to.go[,15]<0.1,]
genes.fdr.sort.down.to.go<- as.character(fdr.sort.down.to.go[,1])
write.table(genes.fdr.sort.down.to.go,file=paste(outpath,"ConceptGen-Input-Genes.txt",sep=""),sep="\t",col.names=FALSE,row.names=FALSE,quote=F)


